# TQMa95xxSA

This README contains some useful information for TQMa95xxSA on MB-SMARC-2

[[_TOC_]]

## Variants

* TQMa95xxSA REV.010x

## Version information for software components

See [here](./README.SoftwareVersions.md) for the software base versions of atf,
bootloader and Linux kernel.

## Supported machine configurations

See top level README.md for configurations usable as MACHINE.

## Supported Features

### U-Boot

| Feature                                          |                             |
| :----------------------------------------------- | :-------------------------: |
| **RAM configs**                                  |                             |
| TQMa95xxSA                                       |                             |
|                                                  |                             |
| CPU variants                                     |                             |
| Fuses                                            |                             |
| GPIO                                             |                             |
| I2C                                              |                             |
| **QSPI**                                         |                             |
| Read                                             |                             |
| Write                                            |                             |
| Boot                                             |                             |
| **e-MMC / SD-Card**                              |                             |
| Read                                             |                             |
| Write                                            |                             |
| Boot                                             |                             |
| **USB**                                          |                             |
| USB 2.0 Dual Role                                |                             |
| USB 3.0 (Hub on TQMa8XxS)                        |                             |
| **ENET (GigE via Phy on TQMa8XxS)**              |                             |
| ENET 0                                           |                             |
| ENET 1                                           |                             |
| **Bootstreams**                                  |                             |
| FlexSPI                                          |                             |
| SD / e-MMC                                       |                             |
| UUU / mfgtool                                    |                             |

**TODO or not tested / supported**


### Linux

| Feature                                          |          fslc-6.6           |
| :----------------------------------------------- | :-------------------------: |
| **RAM configs**                                  |                             |
| TQMa95xxSA                                       |                             |
|                                                  |                             |
| CPU variants                                     |                             |
| Fuses / OCRAM                                    |                             |
| speed grade                                      |                             |
| **UART**                                         |                             |
| console on LPUART7 (X20)                         |             x               |
| LPUART1 on Cortex-M33 (X39)                      |             x               |
| **GPIO**                                         |                             |
| SMARC GPIO pins                                  |             x               |
| **I2C**                                          |                             |
| Temperature Sensors                              |             x               |
| RTC                                              |             x               |
| EEPROMS                                          |             x               |
| **ENET (GigE via Phy on TQMa93xxSA)**            |                             |
| ENET 0                                           |             x               |
| ENET 1                                           |             x               |
| **USB**                                          |                             |
| USB 2.0 Dual Role (X3/X4)                        |                             |
| USB 3.0 (Hub on TQMa93xxSA) (X7/X8)              |             x               |
| **QSPI NOR**                                     |                             |
| Read with 1-4-4 SDR                              |             x               |
| PP / Erase with 1-1-4 SDR                        |             x               |
| **Graphic**                                      |                             |
| GPU                                              |             x               |
| VPU                                              |             x               |
| **Display**                                      |                             |
| LVDS                                             |             x               |
| **Audio**                                        |                             |
| Line IN / Line Out (X17, X18)                    |             x               |
| **PCIe**                                         |                             |
| mini-PCIe on MB-SMARC-2 (X44)                    |             x               |
| PCIe x1 on MB-SMARC-2 (X22)                      |             x               |
| **CAN-FD**                                       |                             |
| CAN-FD                                           |             x               |
| **SPI**                                          |                             |
| SPI user space device on all CS (X24)            |             x               |
| **PWM**                                          |                             |
| PWM in TPM3                                      |             x               |
| **CPU/PMIC thermal sensors**                     |                             |
| via thermal zone                                 |             x               |
| **Cortex M7**                                    |                             |
| examples running from TCM                        |                             |
| use UART as debug console                        |                             |

**TODO or not tested with new BSP**


## Known Issues

* RTC wakeup does not work
* Following GPIOs are not accessible from Cortex-A:
  * GPIO7
  * GPIO8
  * GPIO9
* GPIO13 is not connected on MB-SMARC-2
* SMARC-2 HDA/I2S2 interface not supported
* USB
  * Port USB3 (X3 on MB-SMARC-2) is host only. Do only use a matching adapter
    on MB-SMARC-2
  * USB0 (X4 on MB-SMARC-2) currently only supports USB device mode, without overcurrent detection (!)
* Boot from SPI-NOR is currently unsupported

## Artifacts

Artifacs can be found at the usual locations for bitbake:
`${TMPDIR}/deploy/images/${MACHINE}`

* imx95-tqma9596sa\*.dtb: device tree blobs for TQMa95xxSA
* Image: Linux kernel image
* \*.wic: SD / e-MMC system image
* \*.rootfs.tar.gz: RootFS archive (NFS root etc.)
* imx-boot-${MACHINE}-sd.bin-flash\_a55: boot stream for SD / e-MMC

<!-- FIXME: update list -->

## Boot DIP Switches

_Note:_

* DIP S3 is for Boot Mode.

### SD Card

| DIP S3  | 1 | 2 | 3 | 4 |
| ------- | - | - | - | - |
| ON      |   | x | x |   |
| OFF     | x |   |   | x |

### e-MMC

| DIP S3  | 1 | 2 | 3 | 4 |
| ------- | - | - | - | - |
| ON      | x |   |   |   |
| OFF     |   | x | x | x |

### FLEXSPI

| DIP S3  | 1 | 2 | 3 | 4 |
| ------- | - | - | - | - |
| ON      | x | x |   |   |
| OFF     |   |   | x | x |

### Serial Downloader

| DIP S3  | 1 | 2 | 3 | 4 |
| ------- | - | - | - | - |
| ON      |   |   |   | x |
| OFF     | x | x | x |   |

## Boot device initialisation and update

See [here](./README.imx-arm64.BootMedia.md) for detailed information how to write a
bootstream image and bootloader support for updating the bootstream.

## Use UUU Tool

See [here](./README.imx-arm64.UUU.md) for details about using Serial Download mode and UUU.

Note: UUU is currently unsupported

## Howto

### Test sleepmode and wakeup

Use rtc0 (external RTC on TQMa95xxSA module) or rtc1 (RTC in CPU SNVS domain) to wakeup after 20 seconds:

```
RTC=rtc0
echo enabled > /sys/class/rtc/${RTC}/device/power/wakeup
echo 0 > /sys/class/rtc/${RTC}/wakealarm
echo +20 > /sys/class/rtc//${RTC}/wakealarm
echo mem > /sys/power/state
```

### Display Support

Each Display can be used on its own by using the corresponding device tree.
To allow reusage, the support for each display is separated in a dtsi fragment.

| Interface       | Device tree                                      | Type               |
|-----------------|--------------------------------------------------|--------------------|
| LVDS0           | imx95-tqma9596sa-mb-smarc-2-lvds-tm070jvhg33.dtb | Tianma TM070JVHG33 |

Please note manual for backlight power supply. For MB-SMARC-2 you can bridge
X14 pin 1 and 2 to provide 12V.

### CAN

#### Troubleshooting

In case of problems first check the bus termination:

| Interface | Connector | DIP         |
| --------- | --------- | ----------- |
| CAN0      | X29       | `TERM CAN0` |
| CAN1      | X30       | `TERM CAN1` |

#### Enable without CAN-FD

CAN1/2 should be enabled and configured by default when using with
MB-SMARC-2 and meta-tq / systemd. CAN-FD is disabled by default, as the
transceiver only supports bitrates up to 1MBit/s.
While technically possible using CAN-FD with 1MBit/s, the non-datarate
has to be lowered accordingly.

Configure CAN1/2 per commandline without CAN-FD:
```
CANIF="can[0,1]"
ip link set ${CANIF} up type can bitrate 500000 fd off
```

To (permanently) configure CAN1/2 in systemd network file, set in files
* /lib/systemd/network/20-can0.network
* /lib/systemd/network/20-can1.network

`FDMode=no` to disable CAN-FD.

#### Enable CAN-FD

CAN1/2 should be enabled (without CAN-FD) and configured by default when using with
MB-SMARC-2 and meta-tq / systemd.

To enable CAN-FD the following command can be used, if using a carrier board with
FD capable transceiver:

```
CANIF="can[0,1]"
ip link set ${CANIF} up type can bitrate 500000 sample-point 0.75 dbitrate 1000000 dsample-point 0.8 fd on
```

To (permanently) configure CAN1/2 in systemd network file, set in files
* /lib/systemd/network/20-can0.network
* /lib/systemd/network/20-can1.network

`FDMode=yes` to disable CAN-FD.

**Note**: There is an absolute hardware limit on the bitrate of 1MBit/s, independently from CAN-FD.
When CAN-FD is enabled, the non-datarate needs to be lower than the datarate. The Linux kernel will emit a warning if it is deemed the `brp` setting do not match.

### Cortex M7

<!-- TODO -->

### Access U-Boot environment from Linux

See [U-Boot environment tools](README.libubootenv.md).

## Support Wiki

See [TQ Embedded Wiki for TQMa8XxS](https://support.tq-group.com/en/arm/tqma95xxsa)
